/**************************************************************************
*    Sensor.c
*    Temperature and Pressure sensor Drivers
**************************************************************************/
#include "Sensors.h"
#include "PressureClick_resources.h"

//--------------------------------------------------------------------------
// Definitions for Pressure Sensor
//--------------------------------------------------------------------------
#define _REF_P_XL 0x08                //  Reference pressure (LSB)
#define _REF_P_L 0x09                 //  Reference pressure (middle)
#define _REF_P_H 0x0A                 //  Reference pressure  (MSB)

#define _WHO_AM_I 0x0F                //  Device identification

#define _RES_CONF 0x10                //  Pressure resolution

#define _CTRL_REG1 0x20               //  Control register 1
#define _CTRL_REG2 0x21               //  Control register 2
#define _CTRL_REG3 0x22               //  Control register 3

#define _INT_CFG_REG 0x23             //  Interrupt configuration
#define _INT_SOURCE_REG 0x24          //  Interrupt source

#define _THS_P_LOW_REG 0x25           //  Treshold pressure (LSB)
#define _THS_P_HIGH_REG 0x26          //  Treshold pressure (MSB)

#define _STATUS_REG 0x27              //  Status register

#define _PRESS_POUT_XL_REH 0x28       //  Pressure data (LSB)
#define _PRESS_OUT_L   0x29           //  Pressure data (middle)
#define _PRESS_OUT_H   0x2A           //  Pressure data (MSB)

#define _TEMP_OUT_L 0x2B              //  Temperature data (LSB)
#define _TEMP_OUT_H 0x2C              //  Temperature data (MSB)

#define _AMP_CTRL 0x30                //  Analog front ned control


#define _PRESSURE_W_ADDRESS  0xBA     //  BA   //alt 0xB8
#define _PRESSURE_R_ADDRESS  0xBB     //  BB   //alt 0xB9

//------------------------------------------------------------------------
// variables used for MCP9700A Temp sensor
//------------------------------------------------------------------------
const VREF = 3300 ;

//------------------------------------------------------------------------
// variables used for LSP331AP Pressure Sensor
//-------------------------------------------------------------------------
char PressureoutputStr[14];
char OldPressureoutputStr[14];
char TemperatureStr[15];
char OldTemperatureStr[15];
short address,buffer;
long int  p_value = 0;
long int disp_val = 0;
long int Old_disp_val = 0;
float temp_disp_val = 0;
float Old_temp_disp_val = 0;
int tempor ;
float t_value = 0;
char *res;
//-------------------------------------------------------------------------






//-------------------------------------------------------------------------
void InitSensors()
{
  AD1PCFG.B8 = 0;
  TRISB8_bit = 1;
  AD1PCFG.B9 = 0;
  TRISB9_bit = 1;
  // ADC Init
  ADC1_Init();                        // Initialize ADC

}
//------------------------------------------------------------------------
// Use this function to tread the MMB onboard Temp sensor
static void Get_Temperature()
{
  volatile unsigned long temp;
 // float temp2;
 // read temperature and display it in celsius degrees
  temp = ADC1_Get_Sample(9);
  temp = (unsigned long)(VREF * temp) / 1028;
  temp = temp - 500;

}
//-------------------------------------------------------------------------
// display the temperature
void DisplayTemperature(unsigned long temp)
{
  asm nop;



}
//--------------------------------------------------------------------------
/*initiate I2C communication on Miccrocontroler - Pressure Click
 write one byte through I2C, onto memory address on the Pressure Click */
//--------------------------------------------------------------------------
void Pressure_Write(unsigned short address, unsigned short value) 
{
  I2C2_Start();                   // issue I2C start signal
  I2C2_Write(_PRESSURE_W_ADDRESS);// send byte via I2C  (device address + W)
  I2C2_Write(address);            // send byte (address of the location)
  I2C2_Write(value);              // send data (data to be written)
  I2C2_Stop();                    // issue I2C stop signal
}
//----------------------------------------------------------------------------
//read the measured data from Pressure Click, through I2C communication.
//----------------------------------------------------------------------------
unsigned short Pressure_Read(unsigned short address) 
 {
  unsigned short tmp = 0;

  I2C2_Start();                      // issue I2C start signal
  I2C2_Write(_PRESSURE_W_ADDRESS);   // send byte via I2C (device address + W)
  I2C2_Write(address);               // send byte (data address)

  I2C2_Restart();                    // issue I2C signal repeated start
  I2C2_Write(_PRESSURE_R_ADDRESS);   // send byte (device address + R)
  tmp = I2C2_Read(_I2C_NACK );       // Read the data (NO acknowledge)
  I2C2_Stop();                       // issue I2C stop signal

  return tmp;
}
//-----------------------------------------------------------------------------
// Declaration of function, which will initiate Pressure values
//-----------------------------------------------------------------------------

char Pressure_Init(void) 
 {
  volatile unsigned short temp;
  int err = 0;
  I2C2_Init(100000);       // Initializes I2C with desired clock
  // Initiating the pressure resolution on Pressure Click with writing HEX
  // value in his register ( in this case HEX 0x78)

  Pressure_Write(_RES_CONF, 0x78);
 // Begining of checking of entered value in the register _RES_CONF 
 //of the Pressure Click
  temp = Pressure_Read(_RES_CONF);
  
 //End of checking of entered value in the register _RES_CONF
 // of the Pressure Click
  if (temp != 0x78)
  {
    err++;
  }

  Pressure_Write(_CTRL_REG1, 0x74);
//Begining of checking of entered value in the register _CTRL_REG1
//(Control register 1) of the Pressure Click
  temp = Pressure_Read(_CTRL_REG1);
  
//End of checking of entered value in the register _CTRL_REG1
//Control register 1) of the Pressure Click
  if (temp != 0x74) 
  {
    err++;
  }

  Pressure_Write(_CTRL_REG1, 0xF4);
  
 //Begining of checking of entered value in the register _CTRL_REG1
 //Control register 1) of the Pressure Click
  temp = Pressure_Read(_CTRL_REG1);
  if (temp != 0xF4) {
    err++;
  }
  
 //End of checking of entered value in the register _CTRL_REG1 
 //(Control register 1) of the Pressure Click

  temp = Pressure_read(_WHO_AM_I); 
//Checking the read-only register from Pressure Click, which contains 
//the device ID (Adresse of Device). For LPS331AP, is set to BBh
  if (temp != 0xBB)
  {
    err++;
  }
  return err;
}
//---------------------------------------------------------------------------
//Declaration of function which will read the measuring values of
//pressure from the Pressure Click
//---------------------------------------------------------------------------
long int Pressure_Read_value(void) 
{
 volatile unsigned short low_byte;
 volatile unsigned short mid_byte;
 volatile long int Out_pressure;
  
  
 //values of placed setings from registers _PRESS_OUT_H, from Pressure Click
  Out_pressure = Pressure_Read(_PRESS_OUT_H);
  
//values of placed setings from registers _PRESS_OUT_L, from Pressure Click
  mid_byte = Pressure_Read(_PRESS_OUT_L);
//values of placed setings from registers _PRESS_POUT_XL_REH,from Pressure Click
  low_byte = Pressure_Read(_PRESS_POUT_XL_REH);
// Forming the resulting values of pressure
  Out_pressure = (Out_pressure << 8);
  Out_pressure = (Out_pressure | mid_byte);
  Out_pressure = (Out_pressure << 8);
  Out_pressure = (Out_pressure | low_byte);
  return Out_pressure;
}
//----------------------------------------------------------------------------
// Declaration of function which will read the measuring values of 
//temperature from the Pressure Click
//----------------------------------------------------------------------------
long int Temperature_Read_value(void) 
{
 volatile unsigned short low_byte;
 volatile int Out_temperature;
  
 // Forming the resulting values of temperature
  Out_temperature = Pressure_Read(_TEMP_OUT_H);
  low_byte = Pressure_Read(_TEMP_OUT_L);

  Out_temperature = (Out_temperature << 8);
  Out_temperature = (Out_temperature | low_byte);
  return Out_temperature;
}
//----------------------------------------------------------------------------
void getPressureTemperatureValues(void)
{
  p_value = Pressure_Read_value();
  t_value = (float)Temperature_Read_value();
  disp_val = p_value;
  disp_val >>= 12;
  if(disp_val != Old_disp_val)
  {
   LongWordToStr(disp_val,PressureoutputStr);
   LongWordToStr(Old_disp_val,OldPressureoutputStr);
   Ltrim(PressureoutputStr);
   Ltrim(OldPressureoutputStr);
   TFT_Set_Font(Tahoma28x35_Regular,CL_WHITE,FO_HORIZONTAL);
   TFT_Write_Text(OldPressureoutputStr,300,145);
   TFT_Set_Font(Tahoma28x35_Regular,CL_MAROON,FO_HORIZONTAL);
   TFT_Write_Text(PressureoutputStr,300,145);
   Old_disp_val = disp_val;
 }
 
   temp_disp_val = t_value;
   temp_disp_val = ((temp_disp_val/480) + 42.5) ;
 if(temp_disp_val != Old_temp_disp_val)
 {
  sprintf(TemperatureStr,"%.1f",temp_disp_val);
  sprintf(OldTemperatureStr,"%.1f",Old_temp_disp_val);
  TFT_Set_Font(Tahoma28x35_Regular,CL_WHITE,FO_HORIZONTAL);
  TFT_Write_Text(OldTemperatureStr,114,145);
  TFT_Set_Font(Tahoma28x35_Regular,CL_MAROON,FO_HORIZONTAL);
  TFT_Write_Text(TemperatureStr,114,145);
  Old_temp_disp_val = temp_disp_val;
 }
}
//----------------------------------------------------------------------------
// clear the vars when changing Screens
//----------------------------------------------------------------------------
void ClearSensorVars(void)
{
  p_value = 0;
  disp_val = 0;
  Old_disp_val = 0;
  t_value = 0;
  temp_disp_val = 0;
  Old_temp_disp_val = 0;
}
//----------------------------------------------------------------------------